@@ -1,13 +1,13 @@
 #!/usr/bin/env python3
+from __future__ import print_function
 
-VERSION = '0.4.3'
+VERSION = '0.4.4'
 
 import os
 import sys
 import shutil
 import fileinput
 import functools
-import urllib.request
 import tarfile
 import gzip
 import subprocess
@@ -16,6 +16,13 @@ import argparse
 import string
 import random
 import time
+from contextlib import closing
+
+try:
+    from urllib.request import urlopen, urlretrieve
+except ImportError:
+    from urllib2 import urlopen
+    from urllib import urlretrieve
 
 try:
     width = shutil.get_terminal_size((40, 0)).columns
@@ -96,16 +103,16 @@ def get_args():
 def get_latest_kernel_version(kind='stable'):
     kernel_url = 'https://www.kernel.org/finger_banner'
     search_string = 'The latest {0}'.format(kind)
-    with urllib.request.urlopen(kernel_url) as handle:
+    with closing(urlopen(kernel_url)) as handle:
         for raw_line in handle.readlines():
-            line = str(raw_line, encoding='utf8').rstrip('\n')
+            line = str(raw_line).rstrip('\n')
             if search_string in line:
                 return line.rstrip(' (EOL)').rsplit(' ', 1)[1]
     raise LookupError('Could not find the latest {0} kernel'.format(kind))
 
 
 def get_current_kernel_version():
-    return os.uname().release.split('_', 1)[0]
+    return os.uname()[2].split('_', 1)[0]
 
 
 def get_current_kernel_revision():
@@ -116,10 +123,14 @@ def get_current_kernel_revision():
         return '0'
 
 
+def devnull():
+    return open(os.devnull, 'w')
+
+
 def progress_bar(current, goal):
     marker_width = width - 7
     percent = round(current / goal, 2)
-    mark_count = round(marker_width * percent)
+    mark_count = int(round(marker_width * percent))
     text_bar = '{0:3}% [{1}{2}]'.format(
         int(percent * 100),
         '*' * mark_count,
@@ -191,7 +202,8 @@ class Kernel(object):
         self.config_dir = os.path.abspath(self.config_dir)
 
         for subdir in ['/sources', '/archives']:
-            os.makedirs(self.build_dir + subdir, 0o755, True)
+            if not os.path.isdir(self.build_dir + subdir):
+                os.makedirs(self.build_dir + subdir, 0o755)
 
         raw_configs = [
             x.split('_')
@@ -240,7 +252,7 @@ class Kernel(object):
         else:
             hook = None
         try:
-            urllib.request.urlretrieve(
+            urlretrieve(
                 source,
                 filename=destination,
                 reporthook=hook
@@ -285,7 +297,7 @@ class Kernel(object):
         os.chdir('{0}/sources/linux-{1}'.format(self.build_dir, self.version))
         self.log('Cleaning your kernel tree')
         try:
-            subprocess.call(['make', 'mrproper'], stdout=subprocess.DEVNULL)
+            subprocess.call(['make', 'mrproper'], stdout=devnull())
         except:
             raise EnvironmentError('Failed to clean your kernel tree')
         if self.config_revision == 'current':
@@ -342,7 +354,7 @@ class Kernel(object):
     def make(self, jobs=None, background=True):
         os.chdir('{0}/sources/linux-{1}'.format(self.build_dir, self.version))
         if background:
-            stdout = subprocess.DEVNULL
+            stdout = devnull()
         else:
             stdout = None
         if jobs is None:
